import type { DefineListeners, MazeyFn } from "./typing";
/**
 * Prevent bubbling.
 *
 * Usage:
 *
 * ```javascript
 * const ret1 = cancelBubble(e);
 * ```
 *
 * @category Event
 */
export declare function cancelBubble(e: Event): void;
/**
 * Get event container.
 *
 * Usage:
 *
 * ```javascript
 * const ret = getEventContainer();
 * ```
 *
 * @category Event
 * @hidden
 */
export declare function getDefineListeners(): DefineListeners;
/**
 * Add event.
 *
 * Usage:
 *
 * ```javascript
 * addEvent('test', (e) => {
 *  console.log('test event:', e);
 * });
 * ```
 *
 * @param type
 * @param fn
 * @category Event
 */
export declare function addEvent(type: string, fn: MazeyFn): void;
/**
 * Invoke event.
 *
 * Usage:
 *
 * ```javascript
 * fireEvent('test');
 * ```
 *
 * @param type
 * @category Event
 */
export declare function fireEvent(type: string): void;
/**
 * Alias of `fireEvent`.
 *
 * @hidden
 */
export declare function invokeEvent(type: string): void;
/**
 * Remove event.
 *
 * Usage:
 *
 * ```javascript
 * removeEvent('test');
 * ```
 *
 * @param type
 * @param fn
 * @category Event
 */
export declare function removeEvent(type: string, fn: MazeyFn): void;
//# sourceMappingURL=event.d.ts.map