/**
 * EN: Handle Storage (Keep fit for JSON, it can transfer format automatically).
 *
 * ZH: 存储/获取数据到 sessionStorage/localStorage。
 *
 * Usage:
 *
 * ```javascript
 * setSessionStorage('test', '123');
 * const ret1 = getSessionStorage('test');
 * setLocalStorage('test', '123');
 * const ret2 = getLocalStorage('test');
 * console.log(ret1, ret2);
 *
 * // or package in usage
 * const projectName = 'mazey';
 * function mSetLocalStorage (key, value) {
 *   return setLocalStorage(`${projectName}_${key}`, value);
 * }
 *
 * function mGetLocalStorage (key) {
 *   return getLocalStorage(`${projectName}_${key}`);
 * }
 * ```
 *
 * Output:
 *
 * ```text
 * 123 123
 * ```
 *
 * @param {string} key 键
 * @returns {void} 返回值
 * @category Store
 */
export declare function setSessionStorage<T>(key: string, value?: T | null): void;
/**
 * EN: Handle Storage (Keep fit for JSON, it can transfer format automatically).
 *
 * ZH: 存储/获取数据到 sessionStorage/localStorage。
 *
 * Usage:
 *
 * ```javascript
 * setSessionStorage('test', '123');
 * const ret1 = getSessionStorage('test');
 * setLocalStorage('test', '123');
 * const ret2 = getLocalStorage('test');
 * console.log(ret1, ret2);
 *
 * // or package in usage
 * const projectName = 'mazey';
 * function mSetLocalStorage (key, value) {
 *   return setLocalStorage(`${projectName}_${key}`, value);
 * }
 *
 * function mGetLocalStorage (key) {
 *   return getLocalStorage(`${projectName}_${key}`);
 * }
 * ```
 *
 * Output:
 *
 * ```text
 * 123 123
 * ```
 *
 * @param {string} key 键
 * @returns {any} 返回值
 * @category Store
 */
export declare function getSessionStorage<T>(key: string): T | null;
/**
 * EN: Handle Storage (Keep fit for JSON, it can transfer format automatically).
 *
 * ZH: 存储/获取数据到 sessionStorage/localStorage。
 *
 * Usage:
 *
 * ```javascript
 * setSessionStorage('test', '123');
 * const ret1 = getSessionStorage('test');
 * setLocalStorage('test', '123');
 * const ret2 = getLocalStorage('test');
 * console.log(ret1, ret2);
 *
 * // or package in usage
 * const projectName = 'mazey';
 * function mSetLocalStorage (key, value) {
 *   return setLocalStorage(`${projectName}_${key}`, value);
 * }
 *
 * function mGetLocalStorage (key) {
 *   return getLocalStorage(`${projectName}_${key}`);
 * }
 * ```
 *
 * Output:
 *
 * ```text
 * 123 123
 * ```
 *
 * @param {string} key 键
 * @returns {void} 返回值
 * @category Store
 */
export declare function setLocalStorage<T>(key: string, value?: T | null): void;
/**
 * EN: Handle Storage (Keep fit for JSON, it can transfer format automatically).
 *
 * ZH: 存储/获取数据到 sessionStorage/localStorage。
 *
 * Usage:
 *
 * ```javascript
 * setSessionStorage('test', '123');
 * const ret1 = getSessionStorage('test');
 * setLocalStorage('test', '123');
 * const ret2 = getLocalStorage('test');
 * console.log(ret1, ret2);
 *
 * // or package in usage
 * const projectName = 'mazey';
 * function mSetLocalStorage (key, value) {
 *   return setLocalStorage(`${projectName}_${key}`, value);
 * }
 *
 * function mGetLocalStorage (key) {
 *   return getLocalStorage(`${projectName}_${key}`);
 * }
 * ```
 *
 * Output:
 *
 * ```text
 * 123 123
 * ```
 *
 * @param {string} key 键
 * @returns {void} 返回值
 * @category Store
 */
export declare function getLocalStorage<T>(key: string): T | null;
/**
 * EN: Handle Cookie.
 *
 * ZH: 设置/获取 Cookie。
 *
 * Usage:
 *
 * ```javascript
 * setCookie('test', '123', 30, 'example.com'); // key value day domain
 * const ret = getCookie('test');
 * console.log(ret);
 * ```
 *
 * Output:
 *
 * ```text
 * 123
 * ```
 *
 * @category Store
 */
export declare function getCookie(name: string): string;
/**
 * EN: Handle Cookie.
 *
 * ZH: 设置/获取 Cookie。
 *
 * Usage:
 *
 * ```javascript
 * setCookie('test', '123', 30, 'example.com'); // key value day domain
 * const ret = getCookie('test');
 * console.log(ret);
 * ```
 *
 * Output:
 *
 * ```text
 * 123
 * ```
 *
 * @category Store
 */
export declare function setCookie(name: string, value: string, days?: number, domain?: string): void;
/**
 * Delete a cookie by name.
 *
 * Usage:
 *
 * ```javascript
 * const ret = removeCookie('test');
 * console.log(ret);
 * ```
 *
 * Output:
 *
 * ```text
 * true
 * ```
 *
 * @param name - The name of the cookie to delete.
 * @returns `true` if the cookie was deleted successfully, `false` otherwise.
 * @category Store
 */
export declare function removeCookie(name: string): boolean;
/**
 * Alias of `removeCookie`.
 *
 * @hidden
 */
export declare function delCookie(name: string): void;
//# sourceMappingURL=store.d.ts.map